@@ -13,6 +13,7 @@ from .forms import RESTValidationError
 from .exception import RESTException
 from .utils.compatibility import get_last_parent_pk_field_name
 from .utils.helpers import ModelIterableIteratorHelper
+from .response import HeadersResponse
 
 
 def _get_attr(obj, attr):
@@ -25,170 +26,116 @@ def _get_attr(obj, attr):
 
 class BasePaginator:
 
-    def __init__(self, qs, request):
-        self.qs = qs
-        self.request = request
-
-    @property
-    def page_qs(self):
+    def get_response(self, qs, request):
         raise NotImplementedError
 
-    @property
-    def headers(self):
-        return {}
-
 
 class OffsetBasedPaginator(BasePaginator):
     """
     REST paginator for list and querysets
     """
 
-    MAX_OFFSET = pow(2, 63) - 1
-    MAX_BASE = 100
+    def __init__(self, max_offset=pow(2, 63) - 1, max_base=100, default_base=20):
+        self.max_offset = max_offset
+        self.max_base = max_base
+        self.default_base = default_base
 
-    def __init__(self, qs, request):
-        super().__init__(qs, request)
-        self.base = self._get_base(request)
-        self.total = self._get_total()
-        self.offset = self._get_offset(request)
-        self.next_offset = self._get_next_offset()
-        self.prev_offset = self._get_prev_offset()
+    def get_response(self, qs, request):
+        base = self._get_base(qs, request)
+        total = self._get_total(qs, request)
+        offset = self._get_offset(qs, request)
+        qs = qs[offset:(offset + base + 1)]
+        next_offset = self._get_next_offset(qs, offset, base, total)
+        prev_offset = self._get_prev_offset(qs, offset, base, total)
 
-    def _get_next_offset(self):
-        if self.total:
-            return self.offset + self.base if self.base and self.offset + self.base < self.total else None
-        else:
-            if not self.base:
-                return None
-            next_offset = self.offset + self.base
-            return next_offset if self.qs[next_offset:next_offset + 1].exists() else None
+        return HeadersResponse(
+            ModelIterableIteratorHelper(qs[:base], qs.model),
+            self.get_headers(total, next_offset, prev_offset)
+        )
 
-    def _get_prev_offset(self):
-        return None if self.offset == 0 or not self.base else max(self.offset - self.base, 0)
-
-    def _get_total(self):
-        if isinstance(self.qs, QuerySet):
-            return self.qs.count()
-        else:
-            return len(self.qs)
-
-    def _get_offset(self, request):
+    def _get_offset(self, qs, request):
         offset = request._rest_context.get('offset', '0')
         if offset.isdigit():
             offset_int = int(offset)
-            if offset_int > self.MAX_OFFSET:
-                raise RESTException(ugettext('Offset must be lower or equal to {}').format(self.MAX_OFFSET))
+            if offset_int > self.max_offset:
+                raise RESTException(ugettext('Offset must be lower or equal to {}').format(self.max_offset))
             else:
                 return offset_int
         else:
             raise RESTException(ugettext('Offset must be natural number'))
 
-    def _get_base(self, request):
+    def _get_base(self, qs, request):
         base = request._rest_context.get('base')
         if not base:
-            return None
+            return self.default_base
         elif base.isdigit():
             base_int = int(base)
-            if base_int > self.MAX_BASE:
-                raise RESTException(ugettext('Base must lower or equal to {}').format(self.MAX_BASE))
+            if base_int > self.max_base:
+                raise RESTException(ugettext('Base must lower or equal to {}').format(self.max_base))
             else:
                 return base_int
         else:
             raise RESTException(ugettext('Base must be natural number or empty'))
 
-    @property
-    def page_qs(self):
-        if self.base is not None:
-            return self.qs[self.offset:(self.offset + self.base)]
+    def _get_total(self, qs, request):
+        if isinstance(qs, QuerySet):
+            return qs.count()
         else:
-            return self.qs[self.offset:]
+            return len(qs)
 
-    @property
-    def headers(self):
+    def _get_next_offset(self, qs, offset, base, total):
+        if total:
+            return offset + base if base and offset + base < total else None
+        else:
+            return offset + base if len(qs) > base else None
+
+    def _get_prev_offset(self, qs, offset, base, total):
+        return None if offset == 0 or not base else max(offset - base, 0)
+
+    def get_headers(self, total, next_offset, prev_offset):
         return {
             k: v for k, v in {
-                'X-Total': self.total,
-                'X-Next-Offset': self.next_offset,
-                'X-Prev-Offset': self.prev_offset,
+                'X-Total': total,
+                'X-Next-Offset': next_offset,
+                'X-Prev-Offset': prev_offset,
             }.items() if v is not None
         }
 
 
 class OffsetBasedPaginatorWithoutTotal(OffsetBasedPaginator):
 
-    def _get_total(self):
+    def _get_total(self, qs, request):
         return None
 
 
+class CursorBasedModelIterableIteratorHelper(ModelIterableIteratorHelper):
+
+    def __init__(self, iterable, model, next):
+        super().__init__(iterable, model)
+        self.next = next
+
+
 class CursorBasedPaginator(BasePaginator):
 
-    MAX_BASE = 100
+    def __init__(self, max_base=100, default_base=20):
+        self.max_base = max_base
+        self.default_base = default_base
 
-    def __init__(self, qs, request):
-        super().__init__(qs, request)
-        self.base = self._get_base(request)
-        self.cursor = self._get_cursor(request)
+    def get_response(self, qs, request):
+        base = self._get_base(request)
+        cursor = self._get_cursor(request)
 
-        self._pk_field_name = get_last_parent_pk_field_name(self.qs.model)
-        self._ordering = self._get_ordering(request, qs)
-        self._next_cursor = None
+        ordering = self._get_ordering(request, qs)
 
-    def _get_base(self, request):
-        base = request._rest_context.get('base')
-        if not base:
-            return None
-        elif base.isdigit():
-            base_int = int(base)
-            if base_int > self.MAX_BASE:
-                raise RESTException(ugettext('Base must lower or equal to {}').format(self.MAX_BIG_INT))
-            else:
-                return base_int
-        else:
-            raise RESTException(ugettext('Base must be natural number or empty'))
+        cursor_based_model_iterable = self._get_paged_qs(qs, ordering, cursor, base)
 
-    def _get_cursor(self, request):
-        return request._rest_context.get('cursor')
+        return HeadersResponse(
+            cursor_based_model_iterable,
+            self.get_headers(cursor_based_model_iterable.next)
+        )
 
-    def _get_ordering(self, request, qs):
-        query_ordering = list(qs.query.order_by) or list(qs.model._meta.ordering)
-
-        ordering = []
-        for order_lookup in query_ordering:
-            if isinstance(order_lookup, OrderBy):
-                ordering.append(
-                    '-' + order_lookup.expression.name if order_lookup.descending else order_lookup.expression.name
-                )
-            else:
-                ordering.append(order_lookup)
-
-        if self._pk_field_name not in ordering:
-            ordering.append(self._pk_field_name)
-
-        return ordering
-
-    def _get_position_from_instance(self, instance):
-        if isinstance(instance, dict):
-            attr = instance[self._pk_field_name]
-        else:
-            attr = getattr(instance, self._pk_field_name)
-        return str(attr)
-
-    def get_page(self, qs):
-        if self.base:
-            results = list(qs[:self.base + 1])
-            page = list(results[:self.base])
-
-            self._next_cursor = self._get_position_from_instance(page[-1]) if len(results) > len(page) else None
-
-            return ModelIterableIteratorHelper(page, qs.model)
-        else:
-            return qs
-
-    def _get_field_name(self, order_lookup):
-        return order_lookup[1:] if order_lookup.startswith('-') else order_lookup
-
-    def _get_page_filter_kwargs(self, current_row):
-        ordering = list(self._ordering)
+    def _get_page_filter_kwargs(self, current_row, ordering):
+        ordering = list(ordering)
 
         args_or = []
         while ordering:
@@ -226,13 +173,18 @@ class CursorBasedPaginator(BasePaginator):
             )
         return reduce(or_, args_or)
 
+    def _get_page(self, qs, base):
+        results = list(qs[:base + 1])
+        page = list(results[:base])
+        next_cursor = self._get_position_from_instance(page[-1]) if len(results) > len(page) else None
+        return CursorBasedModelIterableIteratorHelper(page, qs.model, next=next_cursor)
 
     @property
-    def page_qs(self):
-        qs = self.qs.order_by(*self._ordering)
+    def _get_paged_qs(self, qs, ordering, cursor, base):
+        qs = qs.order_by(*ordering)
 
-        if self.cursor:
-            current_row = get_object_or_none(qs, pk=self.cursor)
+        if cursor:
+            current_row = get_object_or_none(qs, pk=cursor)
             if current_row:
                 qs = qs.filter(self._get_page_filter_kwargs(current_row))
             else:
@@ -240,12 +192,56 @@ class CursorBasedPaginator(BasePaginator):
                     ugettext('Cursor object was not found'),
                     code=ERROR_CODE.PAGINATION)
                 )
-        return self.get_page(qs)
+        return self._get_page(qs, base)
 
-    @property
-    def headers(self):
+    def _get_base(self, request):
+        base = request._rest_context.get('base')
+        if not base:
+            return self.default_base
+        elif base.isdigit():
+            base_int = int(base)
+            if base_int > self.max_base:
+                raise RESTException(ugettext('Base must lower or equal to {}').format(self.max_base))
+            else:
+                return base_int
+        else:
+            raise RESTException(ugettext('Base must be natural number or empty'))
+
+    def _get_cursor(self, request):
+        return request._rest_context.get('cursor')
+
+    def _get_ordering(self, request, qs):
+        pk_field_name = get_last_parent_pk_field_name(qs.model)
+        query_ordering = list(qs.query.order_by) or list(qs.model._meta.ordering)
+
+        ordering = []
+        for order_lookup in query_ordering:
+            if isinstance(order_lookup, OrderBy):
+                ordering.append(
+                    '-' + order_lookup.expression.name if order_lookup.descending else order_lookup.expression.name
+                )
+            else:
+                ordering.append(order_lookup)
+
+        if self._pk_field_name not in ordering:
+            ordering.append(pk_field_name)
+
+        return ordering
+
+    def _get_position_from_instance(self, instance):
+        pk_field_name = get_last_parent_pk_field_name(instance.__class__)
+        if isinstance(instance, dict):
+            attr = instance[pk_field_name]
+        else:
+            attr = getattr(instance, pk_field_name)
+        return str(attr)
+
+    def _get_field_name(self, order_lookup):
+        return order_lookup[1:] if order_lookup.startswith('-') else order_lookup
+
+    def get_headers(self, next_cursor):
         return {
             k: v for k, v in {
-                'X-Next-Cursor': self._next_cursor,
+                'X-Next-Cursor': next_cursor,
             }.items() if v is not None
         }
\ No newline at end of file
